@@ -1,3 +1,4 @@
+
 # This file is just a list of exports and documentation. The source
 # code for this file is in Libpng.xs in the top directory.
 
@@ -50,7 +51,6 @@ our @EXPORT_OK = qw/
 	set_sRGB
 	set_packing
 	set_strip_16
-	DESTROY
 	get_internals
 	set_transforms
 	copy_row_pointers
@@ -89,6 +89,7 @@ write_png_file
 color_type_name
 get_internals
 copy_png
+image_data_diff
 /;
 
 our %EXPORT_TAGS = (
@@ -96,10 +97,12 @@ our %EXPORT_TAGS = (
 );
 
 require XSLoader;
-our $VERSION = '0.31_01';
+our $VERSION = '0.38_01';
 
 XSLoader::load('Image::PNG::Libpng', $VERSION);
 
+use Image::PNG::Const ':all';
+
 # Old undocumented function name
 
 sub read_file
@@ -121,6 +124,9 @@ sub read_png_file
     if ($options{transforms}) {
 	$png->set_transforms ($options{transforms});
     }
+    if ($options{verbosity}) {
+	$png->set_verbosity ($options{verbosity});
+    }
     open my $in, "<:raw", $file_name
         or croak "Cannot open '$file_name' for reading: $!";
     $png->init_io ($in);
@@ -220,17 +226,27 @@ sub copy_png
     if ($options{verbosity}) {
 	$opng->set_verbosity ($options{verbosity});
     }
+    my $strip = $options{strip};
+    my $strip_all;
+    if ($strip) {
+	if ($strip eq 'all') {
+	    $strip_all = 1;
+	}
+    }
     my $valid = $png->get_valid ();
     $opng->set_IHDR ($png->get_IHDR ());
-    for my $chunk (keys %$valid) {
-	if ($chunk eq 'IHDR') {
-	    next;
-	}
-	if ($chunk eq 'IDAT') {
-	    my $rows = get_rows ($png);
-	    $opng->set_rows ($rows);
-	}
-	elsif ($valid->{$chunk}) {
+    my $rows = $png->get_rows ();
+    $opng->set_rows ($rows);
+
+    # Set PLTE up first because hIST needs it to be set.
+    if ($valid->{PLTE}) {
+	$opng->set_chunk ('PLTE', $png->get_chunk ('PLTE'));
+    }
+    if (! $strip_all) {
+	# Make a list of valid chunks excluding IHDR (header), IDAT
+	# (image data), and PLTE (palette).
+	my @valid = grep {!/IHDR|IDAT|PLTE/ && $valid->{$_}} sort keys %$valid;
+	for my $chunk (@valid) {
 	    $opng->set_chunk ($chunk, $png->get_chunk ($chunk));
 	}
     }
@@ -249,6 +265,33 @@ sub height
 }
 
 
+sub image_data_diff
+{
+    my ($file1, $file2) = @_;
+    my $png1 = read_png_file ($file1, transforms => PNG_TRANSFORM_EXPAND);
+    my $png2 = read_png_file ($file2, transforms => PNG_TRANSFORM_EXPAND);
+    my $ihdr1 = $png1->get_IHDR ();
+    my $ihdr2 = $png2->get_IHDR ();
+    my @fields = qw/height width/;
+    for my $field (@fields) {
+	if ($ihdr1->{$field} != $ihdr2->{$field}) {
+	    return "$field differs: $file1: ".
+	    "$ihdr1->{field}; $file2: $ihdr2->{field}";
+	}
+    }
+    my $h = $ihdr1->{height};
+    my $rows1 = $png1->get_rows ();
+    my $rows2 = $png2->get_rows ();
+    for my $r (0..$h - 1) {
+ 	my $row1 = $rows1->[$r];
+	my $row2 = $rows2->[$r];
+	if ($row1 ne $row2) {
+	    return "Row $r of image data differs";
+	}
+    }
+    # No difference.
+    return;
+}
 
 1;
 
